#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/21/20, 11:28 AM
#  License: See LICENSE.txt

import logging
import os

from beets import config

# Get values as: plg_ns['__PLUGIN_NAME__']
plg_ns = {}
about_path = os.path.join(os.path.dirname(__file__), u'about.py')
with open(about_path) as about_file:
    exec(about_file.read(), plg_ns)

__logger__ = logging.getLogger('beets.{plg}'.format(plg=plg_ns['__PLUGIN_NAME__']))


def is_plugin_enabled(plugin_name):
    enabled_plugins = config["plugins"].get() if config["plugins"].exists() else []
    return plugin_name in enabled_plugins


def get_plugin_config(plugin_name):
    cfg = None
    if config[plugin_name].exists():
        cfg = config[plugin_name]
    return cfg


def say(msg, log_only=True, is_error=False):
    _level = logging.DEBUG
    _level = _level if log_only else logging.INFO
    _level = _level if not is_error else logging.ERROR
    __logger__.log(level=_level, msg=msg)



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/21/20, 11:27 AM
#  License: See LICENSE.txt

import os

from beets.plugins import BeetsPlugin
from confuse import ConfigSource, load_yaml

from beetsplug.autofix.command import AutofixCommand


class AutofixPlugin(BeetsPlugin):
    _default_plugin_config_file_name_ = 'config_default.yml'

    def __init__(self):
        super(AutofixPlugin, self).__init__()
        config_file_path = os.path.join(os.path.dirname(__file__), self._default_plugin_config_file_name_)
        source = ConfigSource(load_yaml(config_file_path) or {}, config_file_path)
        self.config.add(source)

    def commands(self):
        return [AutofixCommand(self.config)]



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 24/03/2020, 16.00
#  License: See LICENSE.txt

import os
import subprocess
import tempfile

from beets import util
from beetsplug import convert
from beetsplug.convert import ConvertPlugin

from beetsplug.autofix import common
from beetsplug.autofix.task import Task


class AudioConversionTask(Task):
    temp_path = None
    plugin = None

    def __init__(self):
        super(AudioConversionTask, self).__init__()
        assert common.is_plugin_enabled("convert"), "The 'convert' plugin is not enabled!"
        self.plugin = ConvertPlugin()

    def setup(self, config, item):
        self.config = config
        self.item = item

        self.temp_path = os.path.join(tempfile.gettempdir(), self.__class__.__name__)
        if not os.path.isdir(self.temp_path):
            os.mkdir(self.temp_path)

    def run(self):
        if not self._item_needs_processing():
            return

        self._say("Converting: {}".format(self.item.evaluate_template("[$format][$bitrate]: $path")), log_only=True)
        orig_path = self.item.get("path")

        command, ext = convert.get_format()
        fd, dest_path = tempfile.mkstemp(util.py3_path(b'.' + ext), dir=self.temp_path)
        os.close(fd)
        dest_path = util.bytestring_path(dest_path)

        try:
            self.plugin.encode(command, orig_path, dest_path)
        except subprocess.CalledProcessError as err:
            self._say("Conversion error: {}".format(err))
            return

        # Must handle item storage and write differently from autofix
        self.item.path = dest_path
        self.item.write()
        self.item.read()
        os.unlink(orig_path)
        self.item.move()
        self.item.store()

    def _item_needs_processing(self):
        answer = False

        plg_cfg = common.get_plugin_config("convert")

        if plg_cfg["format"].exists():
            fmt = plg_cfg["format"].as_str().lower()
            if self.item.get("format").lower() != fmt:
                answer = True

        if plg_cfg["max_bitrate"].exists():
            max_bitrate = plg_cfg["max_bitrate"].as_number() * 1000
            if int(self.item.get("bitrate")) > max_bitrate:
                answer = True

        return answer



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/26/20, 8:14 PM
#  License: See LICENSE.txt

from beetsplug.autofix import common
from beetsplug.autofix.task import Task
from beetsplug.yearfixer import YearFixerPlugin, YearFixerCommand


class YearFixerTask(Task):
    plugin = None
    plugin_config = None
    year_fixed = False

    def __init__(self):
        super(YearFixerTask, self).__init__()
        assert common.is_plugin_enabled("yearfixer"), "The 'yearfixer' plugin is not enabled!"
        self.plugin = YearFixerPlugin()
        self.plugin_config = common.get_plugin_config("yearfixer")

    def setup(self, config, item):
        self.config = config
        self.item = item
        self.year_fixed = False

    def run(self):
        if not self._item_needs_processing():
            return

        cmd: YearFixerCommand = self.plugin.commands()[0]
        cmd.lib = self.lib

        orig_year = self.item.get("year")
        orig_original_year = self.item.get("original_year")
        self._say("Calling yearfixer plugin for item: {}".format(self.item), log_only=True)
        cmd.process_item(self.item)
        if orig_year != self.item.get("year") or orig_original_year != self.item.get("original_year"):
            self.year_fixed = True

    def _item_needs_processing(self):
        answer = False

        if not self.item.get("year") or not self.item.get("original_year"):
            answer = True

        return answer

    def needs_item_store(self):
        return self.year_fixed

    def needs_item_write(self):
        return self.year_fixed



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/25/20, 3:44 PM
#  License: See LICENSE.txt

from confuse import Subview
from beetsplug.autofix import common
from beetsplug.autofix.task import Task

from beetsplug.acousticbrainz import AcousticPlugin

target_map = {
    'average_loudness': {
        'path': 'lowlevel.average_loudness',
        'type': float,
    },
    'bpm': {
        'path': 'rhythm.bpm',
        'type': int,
    },
    'danceable': {
        'path': 'highlevel.danceability.all.danceable',
        'type': float,
    },
    'gender': {
        'path': 'highlevel.gender.value',
        'type': str,
    },
    'genre_rosamerica': {
        'path': 'highlevel.genre_rosamerica.value',
        'type': str,
    },
    'voice_instrumental': {
        'path': 'highlevel.voice_instrumental.value',
        'type': str,
    },
    'mood_acoustic': {
        'path': 'highlevel.mood_acoustic.all.acoustic',
        'type': float,
    },
    'mood_aggressive': {
        'path': 'highlevel.mood_aggressive.all.aggressive',
        'type': float,
    },
    'mood_electronic': {
        'path': 'highlevel.mood_electronic.all.electronic',
        'type': float,
    },
    'mood_happy': {
        'path': 'highlevel.mood_happy.all.happy',
        'type': float,
    },
    'mood_party': {
        'path': 'highlevel.mood_party.all.party',
        'type': float,
    },
    'mood_relaxed': {
        'path': 'highlevel.mood_relaxed.all.relaxed',
        'type': float,
    },
    'mood_sad': {
        'path': 'highlevel.mood_sad.all.sad',
        'type': float,
    }

    # 'chords_changes_rate': types.Float(6),
    # 'chords_key': types.STRING,
    # 'chords_number_rate': types.Float(6),
    # 'chords_scale': types.STRING,
    # 'initial_key': types.STRING,
    # 'key_strength': types.Float(6),
    # 'rhythm': types.Float(6),
    # 'tonal': types.Float(6),
}


class ABDataFetcherTask(Task):
    plugin = None
    plugin_config: Subview = None
    has_new_data = False

    def __init__(self):
        super(ABDataFetcherTask, self).__init__()
        assert common.is_plugin_enabled("acousticbrainz"), \
            "The 'acousticbrainz' plugin is not enabled!"
        self.plugin = AcousticPlugin()
        self.plugin_config = common.get_plugin_config("acousticbrainz")

    def setup(self, config, item):
        self.config = config
        self.item = item
        self.has_new_data = False

    def run(self):
        if not self._item_needs_processing():
            return

        self._say("Getting AB data: {}".format(self.item), log_only=True)

        abdata = self.plugin._get_data(self.item.mb_trackid)
        if not abdata:
            return

        audiodata = self._extract_from_abdata(abdata)
        if not audiodata:
            return

        self._say("Got clean AB data: {}".format(audiodata), log_only=True)
        for attr in audiodata.keys():
            if (audiodata.get(attr) and not self.item.get(attr)) or \
                    self.plugin_config["force"].get(bool):
                setattr(self.item, attr, audiodata.get(attr))
                self.has_new_data = True

    def _extract_from_abdata(self, abdata):
        """extracts data from the low level json file as mapped out in the
        `low_level_targets` configuration key
        """
        data = {}
        for key in target_map.keys():
            try:
                val = self._extract_value_from_abdata(abdata,
                                                      target_map[key])
            except AttributeError:
                val = None

            data[key] = val

        return data

    @staticmethod
    def _extract_value_from_abdata(abdata, target_map_item: Subview):
        path: str = target_map_item["path"]
        value_type = target_map_item["type"]
        path_parts = path.split(".")

        for part in path_parts:
            if not part in abdata:
                raise AttributeError(
                    "No path '{}' found in abdata".format(path))
            abdata = abdata[part]

        return value_type(abdata)

    def _item_needs_processing(self):
        answer = False

        if not self.item.mb_trackid:
            self._say("Item does not have `mb_trackid`. Cannot proceed.",
                      log_only=True)
            return False

        if self.plugin_config["force"].get(bool):
            return True

        # Check attributes listed in the AB plugin
        for fld in target_map.keys():
            if not self.item.get(fld):
                answer = True
                break

        return answer

    def needs_item_store(self):
        return self.has_new_data

    def needs_item_write(self):
        return self.has_new_data



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 24/03/2020, 15.58
#  License: See LICENSE.txt
import time
from os import path

from beetsplug.autofix.task import Task


class MissingFileCheckerTask(Task):
    removed = False

    def __init__(self):
        super(MissingFileCheckerTask, self).__init__()

    def setup(self, config, item):
        self.config = config
        self.item = item
        self.removed = False

    def run(self):
        if not path.isfile(self.item.get("path")):
            self._say("Item was deleted: {}...".format(self.item), log_only=True)
            self.item.remove()
            self.removed = True

    def item_was_removed(self):
        return self.removed



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/21/20, 11:28 AM
#  License: See LICENSE.txt

import importlib
import time
from optparse import OptionParser

from alive_progress import alive_bar
from beets.dbcore.query import FixedFieldSort
from beets.library import Library, Item, parse_query_parts
from beets.ui import Subcommand, decargs
from confuse import Subview
from beetsplug.autofix import common
from beetsplug.autofix.task import Task


class AutofixCommand(Subcommand):
    config: Subview = None
    lib: Library = None
    query = None
    parser: OptionParser = None

    task_ns = "beetsplug.autofix.task"
    task_registry = None

    cfg_max_exec_time = None

    exec_time_start = None

    def __init__(self, config):
        self.config = config

        cfg = self.config.flatten()
        self.cfg_max_exec_time = cfg.get("max_exec_time")

        self.parser = OptionParser(usage='beet {plg} [options] [QUERY...]'.format(
            plg=common.plg_ns['__PLUGIN_NAME__']
        ))

        self.parser.add_option(
            '-m', '--max_exec_time',
            action='store', dest='max_exec_time', type='int',
            default=self.cfg_max_exec_time,
            help=u'[default: {}] interrupt execution after this the number of seconds'.format(
                self.cfg_max_exec_time)
        )

        self.parser.add_option(
            '-v', '--version',
            action='store_true', dest='version', default=False,
            help=u'show plugin version'
        )

        super(AutofixCommand, self).__init__(
            parser=self.parser,
            name=common.plg_ns['__PLUGIN_NAME__'],
            aliases=[common.plg_ns['__PLUGIN_ALIAS__']] if common.plg_ns['__PLUGIN_ALIAS__'] else [],
            help=common.plg_ns['__PLUGIN_SHORT_DESCRIPTION__']
        )

    def func(self, lib: Library, options, arguments):
        self.lib = lib
        self.query = decargs(arguments)

        self.cfg_max_exec_time = options.max_exec_time

        if options.version:
            self.show_version_information()
            return

        self._setup_task_registry()
        self._execute_tasks_for_all_items()

    def _execute_tasks_for_all_items(self):
        self.exec_time_start = int(time.time())
        items = self._retrieve_library_items(self.query)
        done = 0
        self._say("Total number of items: {}".format(len(items)))
        with alive_bar(len(items)) as bar:
            for item in items:
                try:
                    bar(str(item))
                    self._execute_tasks_for_item(item)
                    done += 1
                except RuntimeError as err:
                    self._say(err)
                    break
                except TimeoutError:
                    self._say("Time is up! {} seconds have passed.".format(self.cfg_max_exec_time))
                    break

        self._say("Done.")
        self._say("Elaborated items: {}".format(done))

    def _execute_tasks_for_item(self, item: Item):
        needs_item_store = False
        needs_item_write = False
        items_was_removed = False
        interrupt_requested = False

        for task_name in self.task_registry.keys():
            task = self.task_registry[task_name]["instance"]
            config = self.task_registry[task_name]["config"]
            self.check_timer()

            try:
                task.setup(config, item)
                task.run()
                needs_item_store = task.needs_item_store() | needs_item_store
                needs_item_write = task.needs_item_write() | needs_item_write
                items_was_removed = task.item_was_removed()
                if items_was_removed:
                    break
            except KeyboardInterrupt:
                interrupt_requested = True
                break
            except BaseException as err:
                self._say('Task module error[{}]: {} Skipping.'.format(task_name, err))
                continue

        if not interrupt_requested:
            if items_was_removed:
                return

            if needs_item_store:
                item.store()

            if needs_item_write:
                item.try_write()

        if interrupt_requested:
            raise RuntimeError("Interrupted")

    def _setup_task_registry(self):
        self.task_registry = {}

        task_list = self.config["tasks"].keys()
        for task_name in task_list:
            config = self.config["tasks"][task_name]

            if not config["enabled"].exists() or not config["enabled"].get(bool):
                self._say("Task({}) is not enabled! Skipping.".format(task_name), log_only=True)
                continue

            try:
                task = self.get_task_class_instance(task_name)
            except ModuleNotFoundError as err:
                self._say("Task not found! {} Skipping.".format(err))
                continue
            except NotImplementedError as err:
                self._say("Bad Task({})! {} Skipping.".format(task_name, err))
                continue
            except RuntimeError as err:
                self._say(
                    "Task({}) runtime error! {} Skipping.".format(task_name,
                                                                  err))
                continue

            # Add library to task
            task.set_library(self.lib)

            self.task_registry[task_name] = {
                "instance": task,
                "config": config
            }

        self._say("The following tasks will be executed: {}".format(", ".join(list(self.task_registry.keys()))))

    def get_task_class_instance(self, module_name):
        module_name_ns = '{0}.{1}'.format(self.task_ns, module_name)

        try:
            module = importlib.import_module(module_name_ns)
        except ModuleNotFoundError as err:
            raise RuntimeError("Module load error! {}".format(err))

        task_class = None
        for obj in module.__dict__.values():
            if isinstance(obj, type) and issubclass(obj, Task) and obj != Task:
                task_class = obj
                break

        if not task_class:
            raise NotImplementedError("There is no subclass of Task class in module: {}".format(module_name_ns))

        try:
            instance = task_class()
        except BaseException as err:
            raise RuntimeError("Instance creation error! {}".format(err))

        return instance

    def _retrieve_library_items(self, query=None, model_cls=Item):
        query = [] if query is None else query
        parsed_query, parsed_sort = parse_query_parts(query, model_cls)

        parsed_sort = FixedFieldSort("albumartist", ascending=True)

        self._say("Selection query[{}]: {}".format(query, parsed_query), log_only=True)
        self._say("Ordering[{}]: {}".format(query, parsed_sort), log_only=True)

        return self.lib.items(parsed_query, parsed_sort)

    def check_timer(self):
        current_time = int(time.time())
        if self.cfg_max_exec_time != 0 and current_time - self.exec_time_start > self.cfg_max_exec_time:
            raise TimeoutError("Time up!")

    def show_version_information(self):
        self._say("{pt}({pn}) plugin for Beets: v{ver}".format(
            pt=common.plg_ns['__PACKAGE_TITLE__'],
            pn=common.plg_ns['__PACKAGE_NAME__'],
            ver=common.plg_ns['__version__']
        ), log_only=False)

    @staticmethod
    def _say(msg, log_only=True, is_error=False):
        common.say(msg, log_only, is_error)



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 24/03/2020, 15.58
#  License: See LICENSE.txt

from beets.library import Item
from confuse import Subview
from beetsplug.autofix import common
from beetsplug.autofix.task import Task
from beetsplug.xtractor import XtractorPlugin, XtractorCommand


class XtractorTask(Task):
    plugin = None
    plugin_config: Subview = None

    def __init__(self):
        super(XtractorTask, self).__init__()
        assert common.is_plugin_enabled("xtractor"), "The 'xtractor' plugin is not enabled!"
        self.plugin = XtractorPlugin()
        self.plugin_config = common.get_plugin_config("xtractor")

    def setup(self, config, item: Item):
        self.config = config
        self.item = item

    def run(self):
        if not self._item_needs_processing():
            return

        self._say("Xtracting item: {}".format(self.item), log_only=True)
        cmd: XtractorCommand = self.plugin.commands()[0]
        cmd.cfg_quiet = True
        cmd.run_full_analysis(self.item)

    def _item_needs_processing(self):
        answer = False

        target_maps = ["low_level_targets", "high_level_targets"]

        for map_key in target_maps:
            if answer:
                break
            target_map = self.plugin_config[map_key]
            for fld in target_map:
                if target_map[fld]["required"].exists() and target_map[fld]["required"].get(bool):
                    if not self.item.get(fld):
                        answer = True
                        break

        return answer



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 24/03/2020, 15.57
#  License: See LICENSE.txt

from beetsplug.autofix.task import Task

from beetsplug.autofix import common
from beetsplug.lastgenre import LastGenrePlugin

ROSAMERICA_GENRES = {
    "cla": "classical",
    "dan": "dance",
    "hip": "hip-hop",
    "jaz": "jazz",
    "pop": "pop",
    "rhy": "rhythm and blues",
    "roc": "rock",
    "spe": "speech"
}


class GenreFinderTask(Task):
    plugin = None
    found_new_genre = False

    def __init__(self):
        super(GenreFinderTask, self).__init__()
        assert common.is_plugin_enabled("lastgenre"), "The 'lastgenre' plugin is not enabled!"
        self.plugin = LastGenrePlugin()

    def setup(self, config, item):
        self.config = config
        self.item = item
        self.found_new_genre = False

    def run(self):
        if not self._item_needs_processing():
            return

        self._say("Finding genre for: {}".format(self.item), log_only=True)

        # Try Lastgenre plugin
        genre, src = self.plugin._get_genre(self.item)
        if genre:
            self._say("Got Lastfm genre [genre: {}][source: {}] for item: {}"
                      .format(genre, src, self.item), log_only=True)
            self.item.genre = genre
            self.found_new_genre = True

        # Try `genre_rosamerica` value - better than nothing
        if self.item.get("genre") is None or self.item.get("genre") == "":
            genre_rosamerica = self.item.get("genre_rosamerica")
            if genre_rosamerica in ROSAMERICA_GENRES.keys():
                genre = ROSAMERICA_GENRES[genre_rosamerica]
                self._say("Got Genre ROSAMERICA genre [genre: {}] for item: {}"
                          .format(genre, self.item), log_only=True)
                self.item.genre = genre
                self.found_new_genre = True

        if not self.item.get("genre"):
            self._say("Unable to find genre for item: {}").format(self.item, log_only=True)

    def _item_needs_processing(self):
        answer = False

        genre = self.item.get("genre")
        if genre is None or genre == "":
            answer = True

        return answer

    def needs_item_store(self):
        return self.found_new_genre

    def needs_item_write(self):
        return self.found_new_genre



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 24/03/2020, 15.58
#  License: See LICENSE.txt

from beetsplug.autofix import common
from beetsplug.autofix.task import Task
from beetsplug.zero import ZeroPlugin


class TagCleanerTask(Task):
    plugin = None
    plugin_config = None
    plugin_fields = None
    is_cleaned = False

    def __init__(self):
        super(TagCleanerTask, self).__init__()
        assert common.is_plugin_enabled("zero"), "The 'zero' plugin is not enabled!"
        self.plugin = ZeroPlugin()
        self.plugin_config = common.get_plugin_config("zero")
        self.plugin_fields = self.plugin_config["fields"].get() \
            if self.plugin_config.exists() and self.plugin_config["fields"].exists() else []

    def setup(self, config, item):
        self.config = config
        self.item = item
        self.is_cleaned = False

    def run(self):
        if not self._item_needs_processing():
            return

        self._say("Cleaning tags: {}".format(self.item), log_only=True)
        self.plugin.process_item(self.item)
        self.is_cleaned = True

    def _item_needs_processing(self):
        answer = False

        for fld in self.plugin_fields:
            if self.item.get(fld):
                answer = True
                break

        return answer

    def needs_item_store(self):
        answer = False

        if self.is_cleaned:
            if (self.plugin_config["update_database"].exists()
                and not self.plugin_config["update_database"].get(bool)) \
                    or not self.plugin_config["update_database"].exists():
                answer = True

        return answer



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/21/20, 11:27 AM
#  License: See LICENSE.txt

import pathlib
from setuptools import setup
from distutils.util import convert_path

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

plg_ns = {}
about_path = convert_path('beetsplug/autofix/about.py')
with open(about_path) as about_file:
    exec(about_file.read(), plg_ns)

# Setup
setup(
    name=plg_ns['__PACKAGE_NAME__'],
    version=plg_ns['__version__'],
    description=plg_ns['__PACKAGE_DESCRIPTION__'],
    author=plg_ns['__author__'],
    author_email=plg_ns['__email__'],
    url=plg_ns['__PACKAGE_URL__'],
    license='MIT',
    long_description=README,
    long_description_content_type='text/markdown',
    platforms='ALL',

    include_package_data=True,
    test_suite='test',
    packages=['beetsplug.autofix', 'beetsplug.autofix.task'],

    python_requires='>=3.6',

    install_requires=[
        'beets>=1.4.9',
        'alive-progress',
    ],

    tests_require=[
        'pytest', 'nose', 'coverage',
        'mock', 'six', 'yaml',
    ],

    # Extras needed during testing
    extras_require={
        'tests': [],
    },

    classifiers=[
        'Topic :: Multimedia :: Sound/Audio',
        'License :: OSI Approved :: MIT License',
        'Environment :: Console',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/24/20, 10:19 AM
#  License: See LICENSE.txt
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/24/20, 9:52 AM
#  License: See LICENSE.txt

from abc import ABC
from abc import abstractmethod

from beets.library import Item, Library
from confuse import Subview
from beetsplug.autofix import common


class Task(ABC):
    config: Subview = None
    lib: Library = None
    item: Item = None

    def __init__(self):
        pass

    def set_library(self, lib):
        self.lib = lib

    def _say(self, msg, log_only=False):
        msg = "[{name}]: {msg}".format(name=self.__class__.__name__, msg=msg)
        common.say(msg, log_only)

    @abstractmethod
    def setup(self, config, item):
        raise NotImplementedError("You must implement this method.")

    @abstractmethod
    def run(self):
        raise NotImplementedError("You must implement this method.")

    @staticmethod
    def needs_item_store():
        """Indicates that the library item needs to be stored
        """
        return False

    @staticmethod
    def needs_item_write():
        """Indicates that the library item needs to be written to the media file
        """
        return False

    @staticmethod
    def item_was_removed():
        """Indicates that the library item was removed (so all further tasks should be skipped)
        """
        return False



#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/27/20, 3:52 PM
#  License: See LICENSE.txt

__author__ = u'Adam Jakab'
__email__ = u'adam@jakab.pro'
__copyright__ = u'Copyright (c) 2020, {} <{}>'.format(__author__, __email__)
__license__ = u'License :: OSI Approved :: MIT License'
__version__ = u'0.1.3'
__status__ = u'Kickstarted'

__PACKAGE_TITLE__ = u'Autofix'
__PACKAGE_NAME__ = u'beets-autofix'
__PACKAGE_DESCRIPTION__ = u'A beets plugin to execute repetitive tasks in one go.',
__PACKAGE_URL__ = u'https://github.com/adamjakab/BeetsPluginAutofix'
__PLUGIN_NAME__ = u'autofix'
__PLUGIN_ALIAS__ = None
__PLUGIN_SHORT_DESCRIPTION__ = u'execute repetitive tasks in one go'
